@@ -2,7 +2,7 @@
    +----------------------------------------------------------------------+
    | Zend Engine                                                          |
    +----------------------------------------------------------------------+
-   | Copyright (c) 1998-2015 Zend Technologies Ltd. (http://www.zend.com) |
+   | Copyright (c) Zend Technologies Ltd. (http://www.zend.com)           |
    +----------------------------------------------------------------------+
    | This source file is subject to version 2.00 of the Zend license,     |
    | that is bundled with this package in the file LICENSE, and is        |
@@ -17,8 +17,6 @@
    +----------------------------------------------------------------------+
 */
 
-/* $Id$ */
-
 /* These iterators were designed to operate within the foreach()
  * structures provided by the engine, but could be extended for use
  * with other iterative engine opcodes.
@@ -30,67 +28,64 @@ typedef struct _zend_object_iterator zend_object_iterator;
 
 typedef struct _zend_object_iterator_funcs {
 	/* release all resources associated with this iterator instance */
-	void (*dtor)(zend_object_iterator *iter TSRMLS_DC);
+	void (*dtor)(zend_object_iterator *iter);
 
 	/* check for end of iteration (FAILURE or SUCCESS if data is valid) */
-	int (*valid)(zend_object_iterator *iter TSRMLS_DC);
+	int (*valid)(zend_object_iterator *iter);
 
 	/* fetch the item data for the current element */
-	void (*get_current_data)(zend_object_iterator *iter, zval ***data TSRMLS_DC);
+	zval *(*get_current_data)(zend_object_iterator *iter);
 
 	/* fetch the key for the current element (optional, may be NULL). The key
 	 * should be written into the provided zval* using the ZVAL_* macros. If
 	 * this handler is not provided auto-incrementing integer keys will be
 	 * used. */
-	void (*get_current_key)(zend_object_iterator *iter, zval *key TSRMLS_DC);
+	void (*get_current_key)(zend_object_iterator *iter, zval *key);
 
 	/* step forwards to next element */
-	void (*move_forward)(zend_object_iterator *iter TSRMLS_DC);
+	void (*move_forward)(zend_object_iterator *iter);
 
 	/* rewind to start of data (optional, may be NULL) */
-	void (*rewind)(zend_object_iterator *iter TSRMLS_DC);
+	void (*rewind)(zend_object_iterator *iter);
 
 	/* invalidate current value/key (optional, may be NULL) */
-	void (*invalidate_current)(zend_object_iterator *iter TSRMLS_DC);
+	void (*invalidate_current)(zend_object_iterator *iter);
+
+	/* Expose owned values to GC.
+	 * This has the same semantics as the corresponding object handler. */
+	HashTable *(*get_gc)(zend_object_iterator *iter, zval **table, int *n);
 } zend_object_iterator_funcs;
 
 struct _zend_object_iterator {
-	void *data;
-	zend_object_iterator_funcs *funcs;
-	ulong index; /* private to fe_reset/fe_fetch opcodes */
+	zend_object std;
+	zval data;
+	const zend_object_iterator_funcs *funcs;
+	zend_ulong index; /* private to fe_reset/fe_fetch opcodes */
 };
 
 typedef struct _zend_class_iterator_funcs {
-	zend_object_iterator_funcs  *funcs;
-	union _zend_function *zf_new_iterator;
-	union _zend_function *zf_valid;
-	union _zend_function *zf_current;
-	union _zend_function *zf_key;
-	union _zend_function *zf_next;
-	union _zend_function *zf_rewind;
+	zend_function *zf_new_iterator;
+	zend_function *zf_valid;
+	zend_function *zf_current;
+	zend_function *zf_key;
+	zend_function *zf_next;
+	zend_function *zf_rewind;
 } zend_class_iterator_funcs;
 
-enum zend_object_iterator_kind {
-	ZEND_ITER_INVALID,
-	ZEND_ITER_PLAIN_ARRAY,
-	ZEND_ITER_PLAIN_OBJECT,
-	ZEND_ITER_OBJECT
-};
+typedef struct _zend_class_arrayaccess_funcs {
+	zend_function *zf_offsetget;
+	zend_function *zf_offsetexists;
+	zend_function *zf_offsetset;
+	zend_function *zf_offsetunset;
+} zend_class_arrayaccess_funcs;
 
 BEGIN_EXTERN_C()
 /* given a zval, returns stuff that can be used to iterate it. */
-ZEND_API enum zend_object_iterator_kind zend_iterator_unwrap(zval *array_ptr, zend_object_iterator **iter TSRMLS_DC);
+ZEND_API zend_object_iterator* zend_iterator_unwrap(zval *array_ptr);
 
 /* given an iterator, wrap it up as a zval for use by the engine opcodes */
-ZEND_API zval *zend_iterator_wrap(zend_object_iterator *iter TSRMLS_DC);
+ZEND_API void zend_iterator_init(zend_object_iterator *iter);
+ZEND_API void zend_iterator_dtor(zend_object_iterator *iter);
 
-ZEND_API void zend_register_iterator_wrapper(TSRMLS_D);
+ZEND_API void zend_register_iterator_wrapper(void);
 END_EXTERN_C()
-
-/*
- * Local variables:
- * tab-width: 4
- * c-basic-offset: 4
- * indent-tabs-mode: t
- * End:
- */
