@@ -19,7 +19,6 @@
 
 #include "php.h"
 #include "php_spl.h"
-#include "ext/pcre/php_pcre.h"
 
 extern PHPAPI zend_class_entry *spl_ce_AppendIterator;
 extern PHPAPI zend_class_entry *spl_ce_CachingIterator;
@@ -44,9 +43,19 @@ extern PHPAPI zend_class_entry *spl_ce_SeekableIterator;
 
 PHP_MINIT_FUNCTION(spl_iterators);
 
-PHP_FUNCTION(iterator_apply);
-PHP_FUNCTION(iterator_count);
-PHP_FUNCTION(iterator_to_array);
+
+typedef enum {
+	RIT_LEAVES_ONLY = 0,
+	RIT_SELF_FIRST  = 1,
+	RIT_CHILD_FIRST = 2
+} RecursiveIteratorMode;
+
+#define RIT_CATCH_GET_CHILD CIT_CATCH_GET_CHILD
+
+typedef enum {
+	RTIT_BYPASS_CURRENT = 4,
+	RTIT_BYPASS_KEY	    = 8
+} RecursiveTreeIteratorFlags;
 
 typedef enum {
 	DIT_Default = 0,
@@ -103,59 +112,6 @@ typedef enum {
 	REGIT_MODE_MAX
 } regex_mode;
 
-typedef struct _spl_cbfilter_it_intern {
-	zend_fcall_info       fci;
-	zend_fcall_info_cache fcc;
-	zend_object           *object;
-} _spl_cbfilter_it_intern;
-
-typedef struct _spl_dual_it_object {
-	struct {
-		zval                 zobject;
-		zend_class_entry     *ce;
-		zend_object          *object;
-		zend_object_iterator *iterator;
-	} inner;
-	struct {
-		zval                 data;
-		zval                 key;
-		zend_long            pos;
-	} current;
-	dual_it_type             dit_type;
-	union {
-		struct {
-			zend_long             offset;
-			zend_long             count;
-		} limit;
-		struct {
-			zend_long             flags; /* CIT_* */
-			zend_string          *zstr;
-			zval             zchildren;
-			zval             zcache;
-		} caching;
-		struct {
-			zval                  zarrayit;
-			zend_object_iterator *iterator;
-		} append;
-		struct {
-			zend_long        flags;
-			zend_long        preg_flags;
-			pcre_cache_entry *pce;
-			zend_string      *regex;
-			regex_mode       mode;
-			int              use_flags;
-		} regex;
-		_spl_cbfilter_it_intern *cbfilter;
-	} u;
-	zend_object              std;
-} spl_dual_it_object;
-
-static inline spl_dual_it_object *spl_dual_it_from_obj(zend_object *obj) /* {{{ */ {
-	return (spl_dual_it_object*)((char*)(obj) - XtOffsetOf(spl_dual_it_object, std));
-} /* }}} */
-
-#define Z_SPLDUAL_IT_P(zv)  spl_dual_it_from_obj(Z_OBJ_P((zv)))
-
 typedef int (*spl_iterator_apply_func_t)(zend_object_iterator *iter, void *puser);
 
 PHPAPI int spl_iterator_apply(zval *obj, spl_iterator_apply_func_t apply_func, void *puser);
